@@ -11,6 +11,7 @@ import (
 	"io"
 	"os"
 	"runtime"
+	"runtime/debug"
 	"strings"
 	"sync"
 	"sync/atomic"
@@ -46,10 +47,13 @@ func New() *tailcfg.Hostinfo {
 		Desktop:         desktop(),
 		Package:         packageTypeCached(),
 		GoArch:          runtime.GOARCH,
+		GoArchVar:       lazyGoArchVar.Get(),
 		GoVersion:       runtime.Version(),
+		Machine:         condCall(unameMachine),
 		DeviceModel:     deviceModel(),
 		Cloud:           string(cloudenv.Get()),
 		NoLogsNoSupport: envknob.NoLogsNoSupport(),
+		AllowsUpdate:    envknob.AllowsRemoteUpdate(),
 	}
 }
 
@@ -60,6 +64,7 @@ var (
 	distroName     func() string
 	distroVersion  func() string
 	distroCodeName func() string
+	unameMachine   func() string
 )
 
 func condCall[T any](fn func() T) T {
@@ -72,6 +77,7 @@ func condCall[T any](fn func() T) T {
 
 var (
 	lazyInContainer = &lazyAtomicValue[opt.Bool]{f: ptr.To(inContainer)}
+	lazyGoArchVar   = &lazyAtomicValue[string]{f: ptr.To(goArchVar)}
 )
 
 type lazyAtomicValue[T any] struct {
@@ -133,6 +139,7 @@ const (
 	FlyDotIo        = EnvType("fly")
 	Kubernetes      = EnvType("k8s")
 	DockerDesktop   = EnvType("dde")
+	Replit          = EnvType("repl")
 )
 
 var envType atomic.Value // of EnvType
@@ -220,6 +227,9 @@ func getEnvType() EnvType {
 	if inDockerDesktop() {
 		return DockerDesktop
 	}
+	if inReplit() {
+		return Replit
+	}
 	return ""
 }
 
@@ -307,6 +317,14 @@ func inFlyDotIo() bool {
 	return false
 }
 
+func inReplit() bool {
+	// https://docs.replit.com/programming-ide/getting-repl-metadata
+	if os.Getenv("REPL_OWNER") != "" && os.Getenv("REPL_SLUG") != "" {
+		return true
+	}
+	return false
+}
+
 func inKubernetes() bool {
 	if os.Getenv("KUBERNETES_SERVICE_HOST") != "" && os.Getenv("KUBERNETES_SERVICE_PORT") != "" {
 		return true
@@ -321,6 +339,27 @@ func inDockerDesktop() bool {
 	return false
 }
 
+// goArchVar returns the GOARM or GOAMD64 etc value that the binary was built
+// with.
+func goArchVar() string {
+	bi, ok := debug.ReadBuildInfo()
+	if !ok {
+		return ""
+	}
+	// Look for GOARM, GOAMD64, GO386, etc. Note that the little-endian
+	// "le"-suffixed GOARCH values don't have their own environment variable.
+	//
+	// See https://pkg.go.dev/cmd/go#hdr-Environment_variables and the
+	// "Architecture-specific environment variables" section:
+	wantKey := "GO" + strings.ToUpper(strings.TrimSuffix(runtime.GOARCH, "le"))
+	for _, s := range bi.Settings {
+		if s.Key == wantKey {
+			return s.Value
+		}
+	}
+	return ""
+}
+
 type etcAptSrcResult struct {
 	mod      time.Time
 	disabled bool
