@@ -8,10 +8,8 @@ import (
 	"fmt"
 	"net/netip"
 	"runtime"
-	"sync/atomic"
 	"testing"
 
-	"gvisor.dev/gvisor/pkg/refs"
 	"tailscale.com/ipn"
 	"tailscale.com/ipn/ipnlocal"
 	"tailscale.com/ipn/store/mem"
@@ -52,13 +50,18 @@ func TestInjectInboundLeak(t *testing.T) {
 		t.Fatal("failed to get internals")
 	}
 
+	lb, err := ipnlocal.NewLocalBackend(logf, "logid", new(mem.Store), "", dialer, eng, 0)
+	if err != nil {
+		t.Fatal(err)
+	}
+
 	ns, err := Create(logf, tunWrap, eng, magicSock, dialer, dns)
 	if err != nil {
 		t.Fatal(err)
 	}
 	defer ns.Close()
 	ns.ProcessLocalIPs = true
-	if err := ns.Start(); err != nil {
+	if err := ns.Start(lb); err != nil {
 		t.Fatalf("Start: %v", err)
 	}
 	ns.atomicIsLocalIPFunc.Store(func(netip.Addr) bool { return true })
@@ -84,24 +87,10 @@ func getMemStats() (ms runtime.MemStats) {
 	return
 }
 
-func TestNetstackLeakMode(t *testing.T) {
-	// See the comments in init(), and/or in issue #4309.
-	// Influenced by an envknob that may be useful in tests, so just check that
-	// it's not the oddly behaving zero value.
-	if refs.GetLeakMode() == 0 {
-		t.Fatalf("refs.leakMode is 0, want a non-zero value")
-	}
-}
-
 func makeNetstack(t *testing.T, config func(*Impl)) *Impl {
 	tunDev := tstun.NewFake()
 	dialer := new(tsdial.Dialer)
-	logf := func(format string, args ...any) {
-		if !t.Failed() {
-			t.Helper()
-			t.Logf(format, args...)
-		}
-	}
+	logf := tstest.WhileTestRunningLogger(t)
 	eng, err := wgengine.NewUserspaceEngine(logf, wgengine.Config{
 		Tun:    tunDev,
 		Dialer: dialer,
@@ -125,10 +114,16 @@ func makeNetstack(t *testing.T, config func(*Impl)) *Impl {
 	}
 	t.Cleanup(func() { ns.Close() })
 
-	ns.atomicIsLocalIPFunc.Store(func(netip.Addr) bool { return true })
-	config(ns)
+	lb, err := ipnlocal.NewLocalBackend(logf, "logid", new(mem.Store), "", dialer, eng, 0)
+	if err != nil {
+		t.Fatalf("NewLocalBackend: %v", err)
+	}
 
-	if err := ns.Start(); err != nil {
+	ns.atomicIsLocalIPFunc.Store(func(netip.Addr) bool { return true })
+	if config != nil {
+		config(ns)
+	}
+	if err := ns.Start(lb); err != nil {
 		t.Fatalf("Start: %v", err)
 	}
 	return ns
@@ -268,11 +263,12 @@ func looksLikeATailscaleSelfAddress(addr netip.Addr) bool {
 
 func TestShouldProcessInbound(t *testing.T) {
 	testCases := []struct {
-		name      string
-		pkt       *packet.Parsed
-		setup     func(*Impl)
-		want      bool
-		runOnGOOS string
+		name        string
+		pkt         *packet.Parsed
+		afterStart  func(*Impl) // optional; after Impl.Start is called
+		beforeStart func(*Impl) // optional; before Impl.Start is called
+		want        bool
+		runOnGOOS   string
 	}{
 		{
 			name: "ipv6-via",
@@ -286,7 +282,7 @@ func TestShouldProcessInbound(t *testing.T) {
 				Dst:      netip.MustParseAddrPort("[fd7a:115c:a1e0:b1a:0:7:a01:109]:5678"),
 				TCPFlags: packet.TCPSyn,
 			},
-			setup: func(i *Impl) {
+			afterStart: func(i *Impl) {
 				prefs := ipn.NewPrefs()
 				prefs.AdvertiseRoutes = []netip.Prefix{
 					// $ tailscale debug via 7 10.1.1.0/24
@@ -297,7 +293,8 @@ func TestShouldProcessInbound(t *testing.T) {
 					LegacyMigrationPrefs: prefs,
 				})
 				i.atomicIsLocalIPFunc.Store(looksLikeATailscaleSelfAddress)
-
+			},
+			beforeStart: func(i *Impl) {
 				// This should be handled even if we're
 				// otherwise not processing local IPs or
 				// subnets.
@@ -318,7 +315,7 @@ func TestShouldProcessInbound(t *testing.T) {
 				Dst:      netip.MustParseAddrPort("[fd7a:115c:a1e0:b1a:0:7:a01:109]:5678"),
 				TCPFlags: packet.TCPSyn,
 			},
-			setup: func(i *Impl) {
+			afterStart: func(i *Impl) {
 				prefs := ipn.NewPrefs()
 				prefs.AdvertiseRoutes = []netip.Prefix{
 					// tailscale debug via 7 10.1.2.0/24
@@ -340,7 +337,7 @@ func TestShouldProcessInbound(t *testing.T) {
 				Dst:       netip.MustParseAddrPort("100.101.102.104:22"),
 				TCPFlags:  packet.TCPSyn,
 			},
-			setup: func(i *Impl) {
+			afterStart: func(i *Impl) {
 				prefs := ipn.NewPrefs()
 				prefs.RunSSH = true
 				i.lb.Start(ipn.Options{
@@ -362,7 +359,7 @@ func TestShouldProcessInbound(t *testing.T) {
 				Dst:       netip.MustParseAddrPort("100.101.102.104:22"),
 				TCPFlags:  packet.TCPSyn,
 			},
-			setup: func(i *Impl) {
+			afterStart: func(i *Impl) {
 				prefs := ipn.NewPrefs()
 				prefs.RunSSH = false // default, but to be explicit
 				i.lb.Start(ipn.Options{
@@ -383,7 +380,7 @@ func TestShouldProcessInbound(t *testing.T) {
 				Dst:       netip.MustParseAddrPort("100.101.102.104:4567"),
 				TCPFlags:  packet.TCPSyn,
 			},
-			setup: func(i *Impl) {
+			afterStart: func(i *Impl) {
 				i.ProcessLocalIPs = true
 				i.atomicIsLocalIPFunc.Store(func(addr netip.Addr) bool {
 					return addr.String() == "100.101.102.104" // Dst, above
@@ -400,9 +397,10 @@ func TestShouldProcessInbound(t *testing.T) {
 				Dst:       netip.MustParseAddrPort("10.1.2.3:4567"),
 				TCPFlags:  packet.TCPSyn,
 			},
-			setup: func(i *Impl) {
+			beforeStart: func(i *Impl) {
 				i.ProcessSubnets = true
-
+			},
+			afterStart: func(i *Impl) {
 				// For testing purposes, assume all Tailscale
 				// IPs are local; the Dst above is something
 				// not in that range.
@@ -419,7 +417,12 @@ func TestShouldProcessInbound(t *testing.T) {
 				Dst:       netip.MustParseAddrPort("10.0.0.23:5555"),
 				TCPFlags:  packet.TCPSyn,
 			},
-			setup: func(i *Impl) {
+			beforeStart: func(i *Impl) {
+				// As if we were running on Linux where netstack isn't used.
+				i.ProcessSubnets = false
+				i.atomicIsLocalIPFunc.Store(func(netip.Addr) bool { return false })
+			},
+			afterStart: func(i *Impl) {
 				prefs := ipn.NewPrefs()
 				prefs.AdvertiseRoutes = []netip.Prefix{
 					netip.MustParsePrefix("10.0.0.1/24"),
@@ -428,13 +431,9 @@ func TestShouldProcessInbound(t *testing.T) {
 					LegacyMigrationPrefs: prefs,
 				})
 
-				// As if we were running on Linux where netstack isn't used.
-				i.ProcessSubnets = false
-				i.atomicIsLocalIPFunc.Store(func(netip.Addr) bool { return false })
-
 				// Set the PeerAPI port to the Dst port above.
-				atomic.StoreUint32(&i.peerapiPort4Atomic, 5555)
-				atomic.StoreUint32(&i.peerapiPort6Atomic, 5555)
+				i.peerapiPort4Atomic.Store(5555)
+				i.peerapiPort6Atomic.Store(5555)
 			},
 			want: false,
 		},
@@ -448,30 +447,11 @@ func TestShouldProcessInbound(t *testing.T) {
 			if tc.runOnGOOS != "" && runtime.GOOS != tc.runOnGOOS {
 				t.Skipf("skipping on GOOS=%v", runtime.GOOS)
 			}
-			impl := makeNetstack(t, func(i *Impl) {
-				defer t.Logf("netstack setup finished")
+			impl := makeNetstack(t, tc.beforeStart)
+			if tc.afterStart != nil {
+				tc.afterStart(impl)
+			}
 
-				logf := tstest.WhileTestRunningLogger(t)
-				e, err := wgengine.NewFakeUserspaceEngine(logf, 0)
-				if err != nil {
-					t.Fatalf("NewFakeUserspaceEngine: %v", err)
-				}
-				t.Cleanup(e.Close)
-
-				lb, err := ipnlocal.NewLocalBackend(logf, "logid", new(mem.Store), "", new(tsdial.Dialer), e, 0)
-				if err != nil {
-					t.Fatalf("NewLocalBackend: %v", err)
-				}
-				t.Cleanup(lb.Shutdown)
-				dir := t.TempDir()
-				lb.SetVarRoot(dir)
-
-				i.SetLocalBackend(lb)
-
-				if tc.setup != nil {
-					tc.setup(i)
-				}
-			})
 			got := impl.shouldProcessInbound(tc.pkt, nil)
 			if got != tc.want {
 				t.Errorf("got shouldProcessInbound()=%v; want %v", got, tc.want)
